#include "defines.h"
#include "interrupt.h"
#include "serial.h"
#include "timer.h"
#include "intr.h"
#include "lib.h"

void trush()
{
  while (tstc())
    getc();
}

char *sendcmd(char *cmd)
{
  static char buf[32];
  puts(cmd);
  puts("\n");
  do {
    gets(buf);
  } while (buf[0] == '\0');
  return buf;
}

int get_ls_come() { return !strcmp(sendcmd("LS_COME"), "ON") ? 1 : 0; }
int get_ls_pass() { return !strcmp(sendcmd("LS_PASS"), "ON") ? 1 : 0; }
int get_rs_come() { return !strcmp(sendcmd("RS_COME"), "ON") ? 1 : 0; }
int get_rs_pass() { return !strcmp(sendcmd("RS_PASS"), "ON") ? 1 : 0; }
int get_us_push() { return !strcmp(sendcmd("US_PUSH"), "ON") ? 1 : 0; }
int get_ds_push() { return !strcmp(sendcmd("DS_PUSH"), "ON") ? 1 : 0; }

int passing;

void set_passing()
{
  static int left_passing;
  static int right_passing;
  if (get_ls_come()) left_passing = 1;
  if (get_ls_pass()) left_passing = 0;
  if (get_rs_come()) right_passing = 1;
  if (get_rs_pass()) right_passing = 0;
  passing = left_passing || right_passing;
}

int lighton = 0;

void light()
{
  static int i = 0;
  i = (i + 1) % 4;
  if (i == 0) {
    lighton = 1;
    sendcmd("LIGHTON");
  }
  if (i == 2) {
    lighton = 0;
    sendcmd("LIGHTOFF");
  }
}

void lightoff()
{
  if (lighton) {
    lighton = 0;
    sendcmd("LIGHTOFF");
  }
}

enum {
  STATUS_OPEN,
  STATUS_CLOSE,
  STATUS_OPENING,
  STATUS_CLOSING,
};

int status_open()
{
  if (passing) {
    sendcmd("CLOSE");
    return STATUS_CLOSING;
  }
  return STATUS_OPEN;
}

int status_close()
{
  if (!passing) {
    sendcmd("OPEN");
    return STATUS_OPENING;
  }
  return STATUS_CLOSE;
}

int status_opening()
{
  if (get_us_push()) {
    sendcmd("STOP");
    return STATUS_OPEN;
  }
  if (passing) {
    sendcmd("CLOSE");
    return STATUS_CLOSING;
  }
  return STATUS_OPENING;
}

int status_closing()
{
  if (get_ds_push()) {
    sendcmd("STOP");
    return STATUS_CLOSE;
  }
  return STATUS_CLOSING;
}

typedef int (*func_t)();

func_t func[] = {
  status_open,
  status_close,
  status_opening,
  status_closing,
};

volatile int sleeping = 0;
void timintr(softvec_type_t type, unsigned long sp)
{
  sleeping = 0;
}

void sleep()
{
  sleeping = 1;
  while (sleeping)
    ;
}

void serintr(softvec_type_t type, unsigned long sp)
{
  int status = 0;
  func_t f;

  trush();
  set_passing();
  f = func[status];
  status = f();
}

void mainloop()
{
  softvec_setintr(SOFTVEC_TYPE_SERINTR, serintr);
  softvec_setintr(SOFTVEC_TYPE_TIMINTR, timintr);
  INTR_ENABLE;

  while (1) {
    if (passing)
      light();
    else
      lightoff();

#if 0
    timer_start(0, 10000, 0);
#if 0
    asm volatile("sleep");
#else
    sleep();
#endif
#endif
  }
}

int main(void)
{
  INTR_DISABLE; /* ̵ˤ */

#ifdef SIMULATOR
  extern int bss_start, ebss;

  memset(&bss_start, 0, (long)&ebss - (long)&bss_start);

  /* եȥߥ٥ */
  softvec_init();

  /* ꥢν */
  serial_init(SERIAL_DEFAULT_DEVICE);
#endif

#if 0
  puts("kozos boot succeed!\n");
#endif

  mainloop();

  /* ˤäƤʤ */

  return 0;
}
